/*
 *	Universal Termcap Terminal Interface
 *
 *	(c) 1997 Martin Mares, <mj@ericsson.cz>
 */

extern ulg rows, columns;
extern ulg supported_attrs;
extern int num_colors, num_real_colors;

#define ATTR_UNDERLINE 1
#define ATTR_ITALIC 2
#define ATTR_SHADOW 4
#define ATTR_BRIGHT 8
#define ATTR_DIM 16
#define ATTR_STANDOUT 32
#define ATTR_INVERSE 64
#define ATTR_ALTERNATE 128		/* Don't relocate */

#define AI_UNDERLINE 0
#define AI_ITALIC 1
#define AI_SHADOW 2
#define AI_BRIGHT 3
#define AI_DIM 4
#define AI_STANDOUT 5
#define AI_INVERSE 6
#define AI_ALTERNATE 7

void init_terminal(void);
void cleanup_terminal(void);
void Tcls(void);
void Trefresh(void);
void Tredraw(void);
void Tgoto(int, int);
void Tsetcursor(int);
void Trestrict(int, int, int, int);
void Tunrestrict(void);
void Tputc(int);
void Tputg(int);
void Tputs(char *);
void Tputt(int);
void Tbell(void);
void Tsetattr(ulg, ulg, ulg);
void Tphysattr(ulg, ulg, ulg);
void Tclear(int, int, int, int);
void Tfill(int, int, int, int, word);
void Tshadow(int, int, int, int);
void Tmove(int, int, int, int, int, int);
void Tscroll(int, int, int, int, int, int);
void Tinsert(int, int, int, byte *);
void *Tsmap_alloc(int, int);
void Tsmap_free(void *);
void Tsmap_put(void *, int, int, int, int, int, int);
void Tsmap_get(void *, int, int, int, int, int, int);
ulg Tgetkey(void);
void Tclean(void);

#define CM_FOLLOWMASK 3			/* Tsetcursor parameter */
#define CM_FOLLOW 0
#define CM_FIXED 1
#define CM_SHAPEMASK 12
#define CM_NORMAL 0
#define CM_ENHANCED 4
#define CM_INVISIBLE 8

#define GFX_UL 0
#define GFX_LL 1
#define GFX_UR 2
#define GFX_LR 3
#define GFX_LTEE 4
#define GFX_RTEE 5
#define GFX_BTEE 6
#define GFX_TTEE 7
#define GFX_HLINE 8
#define GFX_VLINE 9
#define GFX_PLUS 10
#define GFX_HILINE 11
#define GFX_LOLINE 12
#define GFX_DIAMOND 13
#define GFX_CKBOARD 14
#define GFX_DEGREE 15
#define GFX_PLMINUS 16
#define GFX_BULLET 17
#define GFX_LARROW 18
#define GFX_RARROW 19
#define GFX_DARROW 20
#define GFX_UARROW 21
#define GFX_BOARD 22
#define GFX_LANTERN 23
#define GFX_BLOCK 24
#define GFX_DUL 25
#define GFX_DLL 26
#define GFX_DUR 27
#define GFX_DLR 28
#define GFX_DLTEE 29
#define GFX_DRTEE 30
#define GFX_DBTEE 31
#define GFX_DTTEE 32
#define GFX_DHLINE 33
#define GFX_DVLINE 34
#define GFX_DPLUS 35
#define GFX_DSPLUS 36
#define GFX_SDPLUS 37
#define GFX_DSLTEE 38
#define GFX_DSRTEE 39
#define GFX_DSBTEE 40
#define GFX_DSTTEE 41
#define GFX_SDLTEE 42
#define GFX_SDRTEE 43
#define GFX_SDBTEE 44
#define GFX_SDTTEE 45
#define GFX_DSUL 46
#define GFX_DSLL 47
#define GFX_DSUR 48
#define GFX_DSLR 49
#define GFX_SDUL 50
#define GFX_SDLL 51
#define GFX_SDUR 52
#define GFX_SDLR 53

/* Control flags */

extern ulg term_control;

#define TEC_NO_CURSOR_OPT 1
#define TEC_MOVE_ALWAYS_REDRAWS 2
#define TEC_SCROLL_ALWAYS_REDRAWS 4
#define TEC_NO_PC_GRAPHICS 8
#define TEC_NO_VT_GRAPHICS 0x10
#define TEC_NO_COLORS 0x20
#define TEC_IMMEDIATE 0x40
#define TEC_NO_TERMIOS_CHECK 0x80
#define TEC_NO_GETWINSZ 0x100
#define TEC_EMULATE_CLS 0x200
#define TEC_FAKE_SCREEN_SIZE 0x400
#define TEC_NO_GOTO_REFRESH 0x800
#define TEC_NO_VCSA 0x1000

/* Key codes */

#define KEY_BASE 0x1000

#define KEY_UPPER_LEFT (KEY_BASE+0)
#define KEY_UPPER_RIGHT (KEY_BASE+1)
#define KEY_CENTER (KEY_BASE+2)
#define KEY_BACKSPACE (KEY_BASE+3)
#define KEY_BACKTAB (KEY_BASE+4)
#define KEY_LOWER_LEFT (KEY_BASE+5)
#define KEY_LOWER_RIGHT (KEY_BASE+6)
#define KEY_DELETE (KEY_BASE+7)
#define KEY_DOWN (KEY_BASE+8)
#define KEY_INSERT_TOGGLE_DUMMY (KEY_BASE+9)
#define KEY_END (KEY_BASE+10)
#define KEY_ENTER (KEY_BASE+11)
#define KEY_EXIT (KEY_BASE+12)
#define KEY_F0 (KEY_BASE+13)
#define KEY_F1 (KEY_BASE+14)
#define KEY_F2 (KEY_BASE+15)
#define KEY_F3 (KEY_BASE+16)
#define KEY_F4 (KEY_BASE+17)
#define KEY_F5 (KEY_BASE+18)
#define KEY_F6 (KEY_BASE+19)
#define KEY_F7 (KEY_BASE+20)
#define KEY_F8 (KEY_BASE+21)
#define KEY_F9 (KEY_BASE+22)
#define KEY_F10 (KEY_BASE+23)
#define KEY_HELP (KEY_BASE+24)
#define KEY_HOME (KEY_BASE+25)
#define KEY_INSERT (KEY_BASE+26)
#define KEY_LEFT (KEY_BASE+27)
#define KEY_NEXT_PAGE (KEY_BASE+28)
#define KEY_PREV_PAGE (KEY_BASE+29)
#define KEY_RIGHT (KEY_BASE+30)
#define KEY_UP (KEY_BASE+31)

#define KEY_ESCAPE (KEY_BASE+100)
#define KEY_UNKNOWN (KEY_BASE+101)
#define KEY_TAB (KEY_BASE+102)

#define KEY_INTERRUPT (KEY_BASE+200)	/* These never get to the user */
#define KEY_QUIT (KEY_BASE+201)
#define KEY_ERASE (KEY_BASE+202)
#define KEY_SUSPEND (KEY_BASE+203)
#define KEY_WERASE (KEY_BASE+204)
#define KEY_EOF (KEY_BASE+205)
#define KEY_REFRESH (KEY_BASE+206)

#define META 0x4000
